From 50ddd5cc3bc37634ac188762ae679a7ff7156815 Mon Sep 17 00:00:00 2001
From: Alyssa Rosenzweig <alyssa@collabora.com>
Date: Tue, 5 Jul 2022 10:17:15 -0400
Subject: [PATCH] nir: exit barrier opts

---
 src/compiler/nir/nir.h              |  3 ++
 src/compiler/nir/nir_opt_barriers.c | 54 +++++++++++++++++++++++++++++
 2 files changed, 57 insertions(+)

diff --git a/src/compiler/nir/nir.h b/src/compiler/nir/nir.h
index 08ffa825b5b..07e2393432b 100644
--- a/src/compiler/nir/nir.h
+++ b/src/compiler/nir/nir.h
@@ -5446,6 +5446,9 @@ bool nir_opt_combine_memory_barriers(nir_shader *shader,
                                      nir_combine_memory_barrier_cb combine_cb,
                                      void *data);
 
+bool
+nir_opt_remove_exit_barriers(nir_shader *shader);
+
 bool nir_opt_combine_stores(nir_shader *shader, nir_variable_mode modes);
 
 bool nir_copy_prop_impl(nir_function_impl *impl);
diff --git a/src/compiler/nir/nir_opt_barriers.c b/src/compiler/nir/nir_opt_barriers.c
index 18c3d790eb9..299b556a842 100644
--- a/src/compiler/nir/nir_opt_barriers.c
+++ b/src/compiler/nir/nir_opt_barriers.c
@@ -83,3 +83,57 @@ nir_opt_combine_memory_barriers(
 
    return progress;
 }
+
+static bool
+is_barrier(nir_instr *instr)
+{
+   if (instr->type != nir_instr_type_intrinsic)
+      return false;
+
+   nir_intrinsic_instr *intr = nir_instr_as_intrinsic(instr);
+
+   switch (intr->intrinsic) {
+   case nir_intrinsic_control_barrier:
+   case nir_intrinsic_scoped_barrier:
+   case nir_intrinsic_memory_barrier:
+   case nir_intrinsic_group_memory_barrier:
+   case nir_intrinsic_memory_barrier_atomic_counter:
+   case nir_intrinsic_memory_barrier_buffer:
+   case nir_intrinsic_memory_barrier_image:
+   case nir_intrinsic_memory_barrier_shared:
+   case nir_intrinsic_memory_barrier_tcs_patch:
+      return true;
+   default:
+      return false;
+   }
+}
+
+/* Remove barriers from end of an exit block */
+static void
+remove_exit_barrier_helper(nir_block *block)
+{
+   if (exec_list_is_empty(&block->instr_list)) {
+      set_foreach(block->predecessors, entry)
+         remove_exit_barrier_helper((nir_block *) entry->key);
+   } else {
+      nir_foreach_instr_reverse_safe(instr, block) {
+         if (is_barrier(instr))
+            nir_instr_remove(instr);
+         else
+            break;
+      }
+   }
+}
+
+/*
+ * Remove barriers at the end of the program. Barriers stall execution of
+ * subsequent instructions, but if there are no subsequent instruction, they are
+ * not useful and can be removed. This may depend on the underlying hardware.
+ */
+bool
+nir_opt_remove_exit_barriers(nir_shader *shader)
+{
+   nir_function_impl *impl = nir_shader_get_entrypoint(shader);
+
+   remove_exit_barrier_helper(impl->end_block);
+}
-- 
2.35.1

